import os
import sys
import shutil
import kart_quickstart


def build_quickstart():
    base = os.path.join("/".join(kart_quickstart.__file__.split("/")[:-1]))
    for i in os.listdir(base):
        if i[:2] != "__":
            if os.path.isdir(os.path.join(base, i)):
                shutil.copytree(os.path.join(base, i), i)
            else:
                shutil.copyfile(os.path.join(base, i), i)
    os.makedirs("collections/posts", exist_ok=True)
    os.makedirs("collections/tags", exist_ok=True)
    os.makedirs("data", exist_ok=True)
    os.makedirs("pages", exist_ok=True)


if __name__ == "__main__":
    if sys.argv[1] == "init":
        build_quickstart()



from kart import Kart
from kart import miners, mappers

kart = Kart()

kart.miners = [
    miners.DefaultPostMiner("posts"),
    miners.DefaultCollectionMiner("tags"),
    miners.DefaultDataMiner(),
    miners.DefaultPageMiner(),
]

kart.mappers = [
    mappers.DefaultBlogMapper(),
    mappers.DefaultPageMapper(),
    mappers.DefaultFeedMapper(),
]

kart.config["name"] = "My blog"
kart.config["base_url"] = ""
kart.config["pagination"] = {"per_page": 5}

kart.run()



from kart import miners, mappers, renderers
from livereload import Server
import os
import shutil
import argparse


class Kart:
    def __init__(self):
        self.miners = [
            miners.DefaultPostMiner("posts"),
            miners.DefaultCollectionMiner("tags"),
            miners.DefaultDataMiner(),
            miners.DefaultPageMiner(),
        ]
        self.mappers = [
            mappers.DefaultBlogMapper(),
            mappers.DefaultPageMapper(),
            mappers.DefaultFeedMapper(),
        ]
        self.renderers = [
            renderers.DefaultSiteRenderer(),
            renderers.DefaultFeedRenderer(),
        ]
        self.config = {}

    def build(self, build_location="_site"):

        self.build_location = build_location

        self.site = {}
        for miner in self.miners:
            self.site.update(miner.collect())
        self.site["config"] = self.config

        # for key, value in self.site.items():
        #     print(key, value)
        # print(self.site)

        self.map = {}
        for mapper in self.mappers:
            self.map.update(mapper.map(self.site))

        # print(self.map)

        os.makedirs(build_location, exist_ok=True)
        for x in os.listdir(build_location):
            if os.path.isdir(os.path.join(build_location, x)):
                shutil.rmtree(os.path.join(build_location, x))
            else:
                os.remove(os.path.join(build_location, x))
        if "static" in os.listdir():
            shutil.copytree("static", os.path.join(build_location, "static"))

        if "root" in os.listdir():
            for top_level_file in os.listdir("root"):
                shutil.copyfile(
                    os.path.join("root", top_level_file),
                    os.path.join(build_location, top_level_file),
                )

        for renderer in self.renderers:
            renderer.url_function = self.url
            renderer.render(
                self.map, self.site, build_location=build_location,
            )

    def url(self, *name):
        name = ".".join(name)
        if not name:
            return ""
        if name in self.map.keys():
            result = self.map[name]["url"]
        elif name + ".1" in self.map.keys():
            result = self.map[name + ".1"]["url"]
        elif "/" in name:
            result = name
        else:
            return ""
        return self.config["base_url"] + result

    def serve(self, port=9000):
        server = Server()
        server.watcher.ignore_dirs("_site")
        server.watch(".", self.build)
        server.serve(root=self.build_location, port=port, host="0.0.0.0")

    def run(self):
        parser = argparse.ArgumentParser()
        parser.add_argument(
            "command", help="command to execute", choices={"build", "serve"}
        )
        parser.add_argument(
            "-p", "--port", help="port to bind to", default=9000, type=int
        )
        parser.add_argument("--production", action="store_true")
        args = parser.parse_args()
        if not args.production:
            self.config["base_url"] = ""
        if args.command == "build":
            self.build()
        if args.command == "serve":
            self.build()
            self.serve(args.port)



from lxml.html.clean import clean_html
import os
import frontmatter
import yaml
from paka import cmark


class DefaultPostMiner:
    def __init__(self, collection_name="posts", model=None, location="collections"):
        self.model = model
        self.collection_name = collection_name
        self.location = location

    def collect(self):
        self.data = []
        for file in os.listdir(os.path.join(self.location, self.collection_name)):
            with open(
                os.path.join(self.location, self.collection_name, file), "r"
            ) as f:
                metadata, content = frontmatter.parse(f.read())
                object = metadata
                if "draft" in object.keys():
                    if object["draft"]:
                        continue
                object["slug"] = file.split(".")[0]
                object["content"] = cmark.to_html(content)
                object["short_content"] = clean_html(object["content"][:1000])
                self.data.append(object)
        self.data.sort(key=lambda x: x["date"])
        self.data.reverse()
        return {self.collection_name: self.data}


class DefaultCollectionMiner:
    def __init__(self, collection_name, model=None, location="collections"):
        self.model = model
        self.collection_name = collection_name
        self.location = location

    def collect(self):
        self.data = []
        for file in os.listdir(os.path.join(self.location, self.collection_name)):
            with open(
                os.path.join(self.location, self.collection_name, file), "r"
            ) as f:
                metadata, content = frontmatter.parse(f.read())
                object = metadata
                object["slug"] = file.split(".")[0]
                object["content"] = cmark.to_html(content)
                self.data.append(object)
        try:
            self.data.sort(key=lambda x: x["index"])
        except Exception:
            pass
        return {self.collection_name: self.data}


class DefaultPageMiner:
    def __init__(self, location="pages"):
        self.location = location

    def collect(self):
        self.data = {}
        for file in os.listdir(self.location):
            with open(os.path.join(self.location, file), "r") as f:
                metadata, content = frontmatter.parse(f.read())
                object = metadata
                object["content"] = cmark.to_html(content)
                self.data[file.split(".")[0]] = object
        return {"pages": self.data}


class DefaultDataMiner:
    def __init__(self, location="data"):
        self.location = location

    def collect(self):
        self.data = {}
        for file in os.listdir(self.location):
            with open(os.path.join(self.location, file), "r") as f:
                self.data[file.split(".")[0]] = yaml.full_load(f.read())
        return {"data": self.data}



def url(name, object):
    pass


class AutomaticMapper:
    def __init__(self):
        self.urls = []

    def map(self, site):
        return {}


class ManualMapper:
    def __init__(self, rules=[]):
        self.rules = rules  # a rule is a function
        self.urls = {}

    def map(self, site):

        for rule in self.rules:
            self.urls.update(rule(site))

        return self.urls


class DefaultFeedMapper:
    def __init__(self, collections=[]):
        self.urls = {}
        self.collections = collections

    def map(self, site):
        return {
            "atom_feeds": {
                "url": "/atom.xml",
                "data": site["posts"],
                "renderer": "feed_renderer",
            }
        }


class DefaultCollectionMapper:
    def __init__(self, collection_name, base_url="", template="collection_item.html"):
        self.urls = {}
        self.default_template = template
        self.base_url = base_url
        self.collection_name = collection_name

    def map(self, site):
        urls = {}
        collection = site[self.collection_name]
        for object in collection:
            urls.update(
                {
                    f"{self.collection_name}.{object['slug']}": {
                        "url": self.base_url
                        + f"/{self.collection_name}/{object['slug']}/",
                        "data": object,
                        "default_template": self.default_template,
                        "renderer": "main_renderer",
                    }
                }
            )

        return urls


class DefaultPageMapper:
    def __init__(self, template="page.html"):
        self.urls = {}
        self.default_template = template

    def map(self, site):
        for slug in site["pages"]:
            page = site["pages"][slug]
            try:
                url = page["url"]
            except:
                url = f"/{slug}/"
                if slug == "index":
                    url = "/"
            self.urls.update(
                {
                    slug: {
                        "url": url,
                        "data": page,
                        "default_template": self.default_template,
                        "renderer": "main_renderer",
                    }
                }
            )
        return self.urls


class DefaultBlogMapper:
    def __init__(self, base_url=""):
        self.urls = {}
        self.base_url = base_url

    def map(self, site):
        self.urls.update(
            DefaultCollectionMapper(
                base_url=self.base_url, collection_name="posts", template="post.html"
            ).map(site)
        )
        self.urls.update(self.blog_index(site))
        self.urls.update(self.tags(site))

        return self.urls

    def blog_index(self, site):
        urls = {}
        posts = site["posts"][1:]

        try:
            per_page = site["config"]["pagination"]["per_page"]
        except:
            per_page = 5
        paginated_posts = [
            posts[x * per_page : (x + 1) * per_page]
            for x in range(len(posts) // per_page + 1)
        ]

        for i, posts in enumerate(paginated_posts, 1):
            url = f"/index/{i}/" if i > 1 else "/"
            if i > 2:
                previous_page_url = f"/index/{i-1}/"
            elif i == 2:
                previous_page_url = "/"
            else:
                previous_page_url = ""
            if i < len(paginated_posts):
                next_page_url = f"/index/{i+1}/"
            else:
                next_page_url = ""

            paginator = {
                "posts": posts,
                "index": i,
                "next_page_url": self.base_url + next_page_url if next_page_url else "",
                "previous_page_url": self.base_url + previous_page_url
                if previous_page_url
                else "",
            }
            data = {
                "title": "blog",
                "paginator": paginator,
            }
            urls.update(
                {
                    f"blog_index.{i}": {
                        "url": self.base_url + url,
                        "data": data,
                        "default_template": "blog_index.html",
                        "renderer": "main_renderer",
                    }
                }
            )

        return urls

    def tags(self, site):
        urls = {}

        if "tags" not in site.keys():
            return urls

        for tag in site["tags"]:

            posts = site["posts"]
            posts = [post for post in posts if tag["slug"] in post["tags"]]

            try:
                per_page = site["config"]["pagination"]["per_page"]
            except:
                per_page = 5
            paginated_posts = [
                posts[x * per_page : (x + 1) * per_page]
                for x in range(len(posts) // per_page + 1)
            ]

            for i, posts in enumerate(paginated_posts, 1):
                url = f"/tags/{tag['slug']}/{i}/" if i > 1 else f"/tags/{tag['slug']}/"
                if i > 2:
                    previous_page_url = f"/tags/{tag['slug']}/{i-1}/"
                elif i == 2:
                    previous_page_url = f"/tags/{tag['slug']}/"
                else:
                    previous_page_url = ""
                if i < len(paginated_posts):
                    next_page_url = f"/tags/{tag['slug']}/{i+1}/"
                else:
                    next_page_url = ""

                paginator = {
                    "posts": posts,
                    "index": i,
                    "next_page_url": self.base_url + next_page_url
                    if next_page_url
                    else "",
                    "previous_page_url": self.base_url + previous_page_url
                    if previous_page_url
                    else "",
                }
                data = {
                    "title": f"Posts tagged {tag['name']}",
                    "tag_slug": tag["slug"],
                    "paginator": paginator,
                }
                urls.update(
                    {
                        f"tags.{tag['slug']}.{i}": {
                            "url": self.base_url + url,
                            "data": data,
                            "default_template": "tag.html",
                            "renderer": "main_renderer",
                        }
                    }
                )

        return urls



from jinja2 import Environment, FileSystemLoader
from feedgen.feed import FeedGenerator
import os
from datetime import timezone, time, datetime


class DefaultSiteRenderer:
    def __init__(
        self, url_function=None, name="main_renderer", template_folder="templates"
    ):
        self.name = name
        self.template_folder = template_folder
        self.url_function = url_function

    def date_to_string(self, date):
        return date.strftime("%b %d, %Y")

    def render(self, map, site, build_location="_site"):
        self.map = map
        env = Environment(loader=FileSystemLoader(self.template_folder))
        for page in map.values():
            if self.name != page["renderer"]:
                continue
            template = ""
            if "template" in page["data"].keys():
                template = page["data"]["template"]
            if not template:
                template = page["default_template"]
            jinja_template = env.get_template(template)
            jinja_template.globals.update(
                url=self.url_function, date_to_string=self.date_to_string
            )
            page["data"]["url"] = page["url"]
            rendered_file = jinja_template.render(page=page["data"], site=site)
            os.makedirs(build_location + page["url"], exist_ok=True)
            with open(build_location + page["url"] + "index.html", "w") as f:
                f.write(rendered_file)


class DefaultFeedRenderer:
    def __init__(self, name="feed_renderer", collections=["posts"]):
        self.name = name
        self.collections = collections
        self.url_function = None

    def render(self, map, site, build_location="_site"):
        base_url = site["config"]["base_url"]
        for name, page in map.items():
            if self.name != page["renderer"]:
                continue
            fg = FeedGenerator()
            fg.title(site["config"]["name"])
            if not base_url:
                fg.id("base_url/")
            else:
                fg.id(base_url)
            fg.link({"href": base_url})
            fg.link({"href": base_url + page["url"], "rel": "self"})

            feed_entries = []

            for collection in self.collections:
                for object in site[collection]:
                    feed_entries.append([collection, object])

            feed_entries.sort(key=lambda x: x[1]["date"])

            for collection, entry in feed_entries:
                fe = fg.add_entry()
                if "title" in entry.keys():
                    fe.title(entry["title"])
                elif "name" in entry.keys():
                    fe.title(entry["name"])
                if "description" in entry.keys():
                    fe.description(entry["description"])
                elif "short_content" in entry.keys():
                    fe.description(entry["short_content"])
                fe.updated(
                    datetime.combine(entry["date"], time(12), tzinfo=timezone.utc)
                )
                fe.id(self.url_function(collection, entry["slug"]))
                fe.link({"href": self.url_function(collection, entry["slug"])})
            fg.atom_file(build_location + page["url"])
